%% SAS_master.m is a script for detecting courtship pulse song amplitude structure (SAS) of Drosophila and subsequently computing summary statistics of SAS
% Before running SAS_master.m the user needs to preprocess song with SongSegmenter.m (published by Arthur et al.). Preprocessing song with SongSegmenter.m results in the generation of four data structures: Data, Params, Pulses, Sines. 
% These data structures need to be saved by the user for accessing when running this script. 
% This script looks for the above mentioned data structures in the path the
% user determines and -- if the path is correct -- will load and process
% them. If the user wishes to process multiple preprocessed files he or she needs to store all files to be processed in one folder.
% This script saves (a) indices and amplitudes of SAS (point 3.) and (b) the summary statistics average duration, avergage amplitude, average peak_position and average slope under the file name and path the user determines under point 0.
%    Copyright 2015 Birgit Brggemeier

%% 0. Determine paths

% path_to_preprocessedfiles = ['/Users/birgitbruggemeier/Desktop/FlySong/SongBox/Muscle_Mutants/C24/Automatic_Segmentation'];
% path_for_saving_results = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/SAS/MuscleControl/'];                          
% resultsname_for_all_files_in_one_folder = 'C24';

% path_to_preprocessedfiles = ['/Users/birgitbruggemeier/Desktop/FlySong/SongBox/Muscle_Mutants/C26/Automatic_Segmentation'];
% path_for_saving_results = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/SAS/MusclePhosMutant/'];      
% resultsname_for_all_files_in_one_folder = 'C26';

% path_to_preprocessedfiles = ['/Users/birgitbruggemeier/Desktop/FlySong/SongBox/Muscle_Mutants/C43/Automatic_Segmentation'];
% path_for_saving_results = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/SAS/MuscleDualMutant/'];      
% resultsname_for_all_files_in_one_folder = 'C43';

% path_to_preprocessedfiles = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/AutomaticSegmentation_PulseFrequency/Melanogaster/'];
% path_for_saving_results = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/SAS/Melanogaster/'];      
% resultsname_for_all_files_in_one_folder = 'mel';

% path_to_preprocessedfiles = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/AutomaticSegmentation_PulseFrequency/Simulans/'];
% path_for_saving_results = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/SAS/Simulans/'];      
% resultsname_for_all_files_in_one_folder = 'sim';

path_to_preprocessedfiles = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/AutomaticSegmentation_PulseFrequency/Yakuba/'];
path_for_saving_results = ['/Users/birgitbruggemeier/Desktop/PublishSAS/MATLAB/Data/SAS/Yakuba/'];      
resultsname_for_all_files_in_one_folder = 'yak';

% species = 'melanogaster';
% species = 'simulans';
species = 'yakuba';

%% 1. Load data processed with SongSegmenter.m (Arthur, et al., 2013)
% I base the analysis of SAS on song data processed with the SongSegmenter
% published by Arthur et al., 2013

cd(path_to_preprocessedfiles);

files = dir( fullfile(pwd,'*.mat')); % make array with file path to Segmented fly songs

files = {files.name};

for ii = 1:length(files)
    SegmentedSong{ii} = load(files{ii});
end

%% 2. Define constants and initialise variables

min_n_peaks = 1; % What is the minimal number of pulse peaks in one SAS. Default is 1, that is no lower bound restriction in the number of pulses.

%% 3. Compute Song Amplitude Structure with SAS_function.m
for ii = 1:length(SegmentedSong)
Fs{ii} = SegmentedSong{ii}.Data.fs;
[aligned_SAS{ii}, aligned_tSAS{ii}, SAS_positive{ii}, tSAS_positive{ii}, SAS_negative{ii}, tSAS_negative{ii}, tmaxSAS{ii}, tminSAS{ii}, I_SAS_start{ii}, I_SAS_end{ii}, I_SAS{ii}, SAS_count{ii}, a_max{ii}, ta_max{ii}, a_min{ii}, ta_min{ii}, pulse_peaks_per_train{ii}, z{ii}] = SAS_Raw_Data(Fs{ii}, SegmentedSong{ii}, min_n_peaks, species);
end

cd(path_for_saving_results);

save(sprintf('SAS_%s.mat', resultsname_for_all_files_in_one_folder), 'aligned_SAS', 'aligned_tSAS', 'SAS_positive', 'tSAS_positive', 'SAS_negative', 'tSAS_negative', 'tmaxSAS', 'tminSAS', 'I_SAS_start', 'I_SAS_end', 'I_SAS', 'SAS_count', 'a_max', 'ta_max', 'a_min', 'ta_min', 'pulse_peaks_per_train', 'z', 'files');

%% 5. calculate summary stats of SAS
for ii = 1:length(SegmentedSong)
[concat_D_SAS{ii}, D_SAS{ii}, D_SAS_std{ii}, D_SAS_se{ii}, concat_A_SAS{ii}, A_SAS{ii}, A_SAS_std{ii}, A_SAS_se{ii}, m_SAS{ii}, concat_M_SAS{ii}, M_SAS{ii}, M_SAS_std{ii}, M_SAS_se{ii}] = SAS_Stats(I_SAS_start{ii}, I_SAS_end{ii}, pulse_peaks_per_train{ii}, tmaxSAS{ii}, Fs{ii}, SAS_positive{ii}, tSAS_positive{ii});
end

%% 6. Control: Is the length of pulse trains affected by muscle mutations?
for ii = 1:length(files)
[ SongLabel{ii}, I_start_NoSong{ii}, I_end_NoSong{ii}, SineTrain{ii}, PulseTrain{ii}, bTrain{ii} , I_start_PulseSong{ii}, I_end_PulseSong{ii}, I_start_SineSong{ii}, I_end_SineSong{ii}] = SongTypeLabeling( SegmentedSong{ii}.Data, SegmentedSong{ii}.Pulses, SegmentedSong{ii}.Sines , species);
duration_pulse_train{ii} = (I_end_PulseSong{ii} - I_start_PulseSong{ii})/Fs{ii};
end

duration_pulse_trains = horzcat(duration_pulse_train{ii});

mean_dur_pulsetrains = mean(duration_pulse_trains);
std_dur_pulsetrains = std(duration_pulse_trains);
se_dur_pulsetrains = std(duration_pulse_trains)/sqrt(length(duration_pulse_trains));

n_pulsetrains = length(duration_pulse_trains);

save(sprintf('SAS_summarystats_%s.mat', resultsname_for_all_files_in_one_folder), 'concat_D_SAS','D_SAS', 'D_SAS_std', 'D_SAS_se', 'concat_A_SAS', 'A_SAS', 'A_SAS_std', 'A_SAS_se', 'm_SAS','concat_M_SAS', 'M_SAS', 'M_SAS_std', 'M_SAS_se', 'mean_dur_pulsetrains', 'std_dur_pulsetrains', 'se_dur_pulsetrains', 'n_pulsetrains');
