%% function for computing SAS - Song Amplitude Structure: Song Amplitude Structure describes the distinct pattern of increasing and decreasing amplitude during bouts of courtship song of the fruit fly Drosophila.
% We assume SAS to be based on muscle properties of Drosophila.
%    Copyright 2015 Birgit Brggemeier

function [aligned_SAS, aligned_tSAS, SAS_positive, tSAS_positive, SAS_negative, tSAS_negative, tmaxSAS, tminSAS, I_SAS_start, I_SAS_end, I_SAS, SAS_count, a_max, ta_max, a_min, ta_min, pulse_peaks_per_train, z] = SAS_Raw_Data(Fs, SegmentedSong, min_n_peaks, species);

fuzzy_I_correction = 0.05; % fuzzy_I_correction constant should be smaller than timelag in function SongTypeLabeling - and large enough to correct for fuzzy identification of Indices (the final decision for the constant is subject to the user)
%% 1. Calculate start and end points of pulse trains



[ SongLabel, I_start_NoSong, I_end_NoSong, SineTrain, PulseTrain, bTrain, I_start_PulseSong, I_end_PulseSong, I_start_SineSong, I_end_SineSong] = SongTypeLabeling( SegmentedSong.Data, SegmentedSong.Pulses, SegmentedSong.Sines, species);

n_pulse_trains = length(I_start_PulseSong);

for ee = 1:n_pulse_trains
    n_pulse_peaks(ee) = length(find(SegmentedSong.Pulses.AmpCull.w0 >= I_start_PulseSong(ee) - Fs*fuzzy_I_correction & SegmentedSong.Pulses.AmpCull.w1 <= I_end_PulseSong(ee) + Fs*fuzzy_I_correction));
    
    if n_pulse_peaks(ee) < min_n_peaks
        pulse_peaks_per_train.start{ee} = [];
        pulse_peaks_per_train.end{ee} = [];
    else
        pulse_peaks_per_train.start{ee} = SegmentedSong.Pulses.AmpCull.w0(find(SegmentedSong.Pulses.AmpCull.w0 >= I_start_PulseSong(ee) - Fs*fuzzy_I_correction & SegmentedSong.Pulses.AmpCull.w1 <= I_end_PulseSong(ee) + Fs*fuzzy_I_correction));
        pulse_peaks_per_train.end{ee} = SegmentedSong.Pulses.AmpCull.w1(find(SegmentedSong.Pulses.AmpCull.w0 >= I_start_PulseSong(ee)  - Fs*fuzzy_I_correction & SegmentedSong.Pulses.AmpCull.w1 <= I_end_PulseSong(ee) + Fs*fuzzy_I_correction));
    end
    
end
% clear off pulse trains with less than the minimal amount of pulse
% peaks min_n_peaks - this step is repeated on point 4 b); Repeated winnowing of variables reduces computation time
n_pulse_trains = n_pulse_trains - sum(cellfun('isempty',pulse_peaks_per_train.start));
n_pulse_peaks(cellfun('isempty',pulse_peaks_per_train.start)) = [];
pulse_peaks_per_train.start(cellfun('isempty',pulse_peaks_per_train.end)) = [];
h_cell = cellfun('isempty',pulse_peaks_per_train.end);
pulse_peaks_per_train.end(h_cell) = [];



clearvars 'SongLabel' 'I_start_NoSong' 'I_end_NoSong' 'bTrain';



%% 2. Pulse

% 2.a) Identify peaks of pulses; the peaks of pulses give the contour of SAC

for ee = 1:n_pulse_trains
    
    
    for ff = 1:n_pulse_peaks(ee)
        
        a_max{ee}(ff) = max(SegmentedSong.Data.d(pulse_peaks_per_train.start{ee}(ff) : pulse_peaks_per_train.end{ee}(ff)));
        a_min{ee}(ff) = min(SegmentedSong.Data.d(pulse_peaks_per_train.start{ee}(ff) : pulse_peaks_per_train.end{ee}(ff)));
        
        data_chunk_h = zeros(1, length(SegmentedSong.Data.d));
        data_chunk_h(pulse_peaks_per_train.start{ee}(ff) : pulse_peaks_per_train.end{ee}(ff)) = SegmentedSong.Data.d(pulse_peaks_per_train.start{ee}(ff) : pulse_peaks_per_train.end{ee}(ff));
        ta_max{ee}(ff) = find(data_chunk_h == a_max{ee}(ff),1, 'first'); % time index of positive amplitude peak
        ta_min{ee}(ff) = find(data_chunk_h == a_min{ee}(ff),1, 'first'); % time index of negative amplitude peak
    end
    
    %clear off pulse trains that do not include at least one pulse peak
    %with the minimal amplitude I expect for pulses
    [rows columns] = size(SegmentedSong.Data.d);
    
    if columns > 1
        SegmentedSong.Data.d = SegmentedSong.Data.d(:,1);
    end
    
    if isempty(find(a_max{ee} > 0))
        n_pulse_trains = n_pulse_trains-1;
        n_pulse_peaks(ee) = 0;
        a_max{ee} = [];
        a_min{ee} = [];
        ta_max{ee} = [];
        ta_min{ee} = [];
        pulse_peaks_per_train.start{ee} = [];
        pulse_peaks_per_train.end{ee} = [];
    end
    
end

n_pulse_peaks = n_pulse_peaks(find(n_pulse_peaks > 0));
h_cell2 = cellfun('isempty',pulse_peaks_per_train.end);
a_max(h_cell2) = [];
a_min(h_cell2) = [];
ta_max(h_cell2) = [];
ta_min(h_cell2) = [];
pulse_peaks_per_train.start(h_cell2) = [];
pulse_peaks_per_train.end(h_cell2) = [];



% 2.b) Divide Pulse Trains into SASs


if isempty(a_max) == 1
    
    n_pulse_trains = 0;
    n_pulse_peaks = [];
    pulse_peaks_per_train.start = [];
    pulse_peaks_per_train.end = [];
    I_a_max_diff = [];
    a_max = [];
    ta_max = [];
    a_min = [];
    ta_min = [];
    I_SAS = [];
    
else
    
    for ee = 1:n_pulse_trains
        
        h_diff = diff(a_max{ee});
        
        a_max_diff{ee} = diff([-1, a_max{ee}]);
        
        I_a_max_diff{ee} = ones(1,length(a_max_diff{ee}));
        I_a_max_diff{ee}(find(a_max_diff{ee} < 0)) = -1;
        %             I_a_max_diff{ii}{ee}(end) = -1; % the end of the song train is always the end of the last SAS in this song train
        
        h = find(I_a_max_diff{ee}<0);
        
        if length(h)== 0
            I_SAS{ee} = length(I_a_max_diff{ee});
        elseif length(h) ==1
            
            if I_a_max_diff{ee}(end)<0
                I_SAS{ee} = h;
            else
                I_SAS{ee} = [h, length(I_a_max_diff{ee})];
            end
        else
            if I_a_max_diff{ee}(end)<0
                I_SAS{ee} = [h(find(h(2:end) - h(1:end-1)>1)),h(end)];
                if isempty(I_SAS{ee}) == 1
                    I_SAS{ee} = h(end);
                end
            else
                I_SAS{ee} = [h(find(h(2:end) - h(1:end-1)>1)),h(end), length(I_a_max_diff{ee})];
                if isempty(I_SAS{ee}) == 1
                    I_SAS{ee} = [h(end), length(I_a_max_diff{ee})];
                end
            end
        end
        
    end
    
    % clear off pulse trains which do not have any SAS
    n_pulse_trains = n_pulse_trains - sum(cellfun('isempty',I_SAS));
    n_pulse_peaks(cellfun('isempty',I_SAS)) = [];
    pulse_peaks_per_train.start(cellfun('isempty', I_SAS)) = [];
    pulse_peaks_per_train.end(cellfun('isempty', I_SAS)) = [];
    I_a_max_diff(cellfun('isempty',I_SAS)) = [];
    a_max(cellfun('isempty', I_SAS)) = [];
    ta_max(cellfun('isempty', I_SAS)) = [];
    a_min(cellfun('isempty', I_SAS)) = [];
    ta_min(cellfun('isempty', I_SAS)) = [];
    
    I_SAS(cellfun('isempty', I_SAS)) = []; % at this point only pulse trains are left, 1which have (1) at least one SAS and (2) SAS of a minimal max amplitude of max_amp>0
end



% calculate start and end indices of SAS and clear off SAS start indices which point to a SAS with less pulse
% peaks than specified by the minimal number of pulse peaks, which
% is min_n_peaks

if isempty(I_SAS) == 1
    I_SAS_start = [];
    I_SAS_end = [];
    
else
    [I_SAS_start, I_SAS_end] = function_SAS_startendpoint(I_SAS, n_pulse_trains, min_n_peaks);
end

% 2.c) Identify index of Maximum of SASs


if isempty(I_SAS_start) == 1
    
    
else
    for ee = 1:n_pulse_trains
        
        for ff = 1:length(I_SAS_start{ee})
            
            SAS_positive{ee}{ff} = a_max{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff));
            tSAS_positive{ee}{ff} = ta_max{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff));
            SAS_negative{ee}{ff} = a_min{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff));
            tSAS_negative{ee}{ff} = ta_min{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff));
            
            tmaxSAS{ee}(ff) = tSAS_positive{ee}{ff}(find(SAS_positive{ee}{ff} == max(SAS_positive{ee}{ff}),1, 'first'));
            tminSAS{ee}(ff) = tSAS_negative{ee}{ff}(find(SAS_negative{ee}{ff} == min(SAS_negative{ee}{ff}),1, 'first'));
            
            % winnow pulse_peaks_per_train, that is clear off
            % pulse_peaks_per_train.start and pulse_peaks_per_train.end
            % indices which are not part of an SAS with the minimal
            % number of peaks  min_n_peaks, as indexed by I_SAS_start
            % and I_SAS_end; do this winnowing only if min_n_peaks >1,
            % i.e. there are potentially SAS peaks to be excluded
            if min_n_peaks > 1;
                if ff == 1
                    pulse_peaks_per_train.start{ee} = pulse_peaks_per_train.start{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff));
                    pulse_peaks_per_train.end{ee} = pulse_peaks_per_train.end{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff));
                else
                    pulse_peaks_per_train.start{ee} = horzcat(pulse_peaks_per_train.start{ee}, pulse_peaks_per_train.start{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff)));
                    pulse_peaks_per_train.end{ee} = horzcat(pulse_peaks_per_train.end{ee}, pulse_peaks_per_train.end{ee}(I_SAS_start{ee}(ff):I_SAS_end{ee}(ff)));
                end
            end
            % 4.d) Align SASs for plotting
            % loop through concatenated SAS Index vector
            
            aligned_tSAS_positive{ee}{ff} = tSAS_positive{ee}{ff} - tmaxSAS{ee}(ff); % Align SASs for plotting
            aligned_tSAS_negative{ee}{ff} = tSAS_negative{ee}{ff} - tmaxSAS{ee}(ff); % Align SASs for plotting
            
        end
    end
end

%% 3. Make Frequency matrix z for plotting z as surf plot
% concatenate all SAS Indices to one vector
ht_pos = horzcat(aligned_tSAS_positive{:});
h2t_pos = horzcat(ht_pos{:});

ht_neg = horzcat(aligned_tSAS_negative{:});
h2t_neg = horzcat(ht_neg{:});


aligned_tSAS = [h2t_pos, h2t_neg];

h_pos = horzcat(SAS_positive{:});
h2_pos = horzcat(h_pos{:});

h_neg = horzcat(SAS_negative{:});
h2_neg = horzcat(h_neg{:});


aligned_SAS = [h2_pos, h2_neg];

xy =[aligned_tSAS' aligned_SAS'];

xy(isnan(xy(:,1)),:) =[];
xNOnan = xy(:,1);
yNOnan = xy(:,2);

n = 30;
xi = linspace(min(xNOnan), max(xNOnan), n);
xi = fliplr(xi);
yi = linspace(min(yNOnan), max(yNOnan), n);

xr = interp1(xi,0.5:numel(xi)-0.5,xNOnan,'nearest');
yr = interp1(yi,0.5:numel(yi)-0.5,yNOnan,'nearest');

%% 4. generate frequency arrary of aligned SAS
z = (accumarray([xr yr] + 0.5, 1, [n n])/length(xNOnan))*100;

%% How many SASs did I analyse?
SAS_count = 0;
for ee = 1:n_pulse_trains
    
    for ff = 1:length(I_SAS_start{ee})
        SAS_count = SAS_count +1;
        
    end
    
end

end % end function