% Calculate (1) mean length of SAS and (2) calculate amplitude of SAS-peak 
% and (3) the mean slope of amplitude increase in SAS
%    Copyright 2015 Birgit Brggemeier

% For (1) Calculate mean length of SAS

function [concat_D_SAS,D_SAS, D_SAS_std, D_SAS_se, concat_A_SAS, A_SAS, A_SAS_std, A_SAS_se, m_SAS, concat_M_SAS, M_SAS, M_SAS_std, M_SAS_se] = SAS_Stats(I_SAS_start, I_SAS_end, pulse_peaks_per_train, tmaxSAS, Fs, SAS_positive, tSAS_positive)

max_duration_SAS = 1500; %ms % we chose this threshold as there were a few extreme outliers to the distribution, which we wanted to exclude


if isempty(pulse_peaks_per_train.start) == 1
    
    D_SAS_h2 = nan;
    concat_D_SAS = nan;
else
    for ee = 1:length(I_SAS_start)
        
        
        D_SAS_h{ee} = (pulse_peaks_per_train.end{ee}(I_SAS_end{ee}) - pulse_peaks_per_train.start{ee}(I_SAS_start{ee}))./(Fs/1000);
        
        D_SAS_h2(ee) =  sum(D_SAS_h{ee})/length(D_SAS_h{ee});
        
        if ee ==1
            concat_D_SAS = D_SAS_h{ee};
        else
            concat_D_SAS = horzcat(concat_D_SAS, D_SAS_h{ee});
        end
    end
end

if isnan(D_SAS_h2)==1
    D_SAS_h3 = nan;
else
    D_SAS_h3 = sum(D_SAS_h2)/length(D_SAS_h2);
end


D_SAS = sum(D_SAS_h3(find(isnan(D_SAS_h3) == 0)))/length(D_SAS_h3(find(isnan(D_SAS_h3) == 0)));

concat_D_SAS_h_outlier = concat_D_SAS;
concat_D_SAS(find(concat_D_SAS_h_outlier > max_duration_SAS)) = [];
concat_D_SAS(find(isnan(concat_D_SAS) == 1)) = [];

D_SAS = sum(concat_D_SAS)/length(concat_D_SAS);
D_SAS_std = std(concat_D_SAS);
D_SAS_se = D_SAS_std/sqrt(length(concat_D_SAS));


% For (2) calculate amplitude of song %

for ee = 1:length(I_SAS_start)
    concat_A_SAS_h1{ee} = horzcat(SAS_positive{ee}{:});
end
concat_A_SAS = horzcat(concat_A_SAS_h1{:});


concat_A_SAS(find(concat_D_SAS_h_outlier > max_duration_SAS)) = [];
concat_A_SAS(find(isnan(concat_A_SAS) == 1)) = [];
if max(concat_A_SAS) >1
    concat_A_SAS(find(concat_A_SAS > 1)) = 1;
end

A_SAS = sum(concat_A_SAS)/length(concat_A_SAS);
A_SAS_std = std(concat_A_SAS);
A_SAS_se = A_SAS_std/sqrt(length(concat_A_SAS));

% For (3) calculate the mean slope of amplitude increase in SAS

% 4.1. identify data points that can be used to calculate increasing slope
% of the positive part of SAS, which needs to meet the following two
% criteria:
% criteria 1: SAS need to have at least two peaks to calculate a slope
% criteria 2: I focus on positive increasing Amplitude only for slope calculation

for ee = 1:length(I_SAS_start) % loop through pulse trains
    for ff = 1:length(I_SAS_start{ee}) % loop through SAS
        if length(SAS_positive{ee}{ff})>=2; % criteria 1: SAS need to have at least two peaks to calculate a slope
            if tmaxSAS{ee}(ff) == tSAS_positive{ee}{ff}(end) % criteria 2: I focus on positive increasing Amplitude only for slope calculation
                Sl_SAS{ee}{ff} = SAS_positive{ee}{ff};
                tSl_SAS{ee}{ff} = tSAS_positive{ee}{ff};
            elseif tmaxSAS{ee}(ff) == tSAS_positive{ee}{ff}(1)
                Sl_SAS{ee}{ff} = [];
                tSl_SAS{ee}{ff} = [];
            else
                Sl_SAS{ee}{ff} = SAS_positive{ee}{ff};
                tSl_SAS{ee}{ff} = tSAS_positive{ee}{ff};
                
                relative_peak_pos = find( tSAS_positive{ee}{ff} == tmaxSAS{ee}(ff));
                
                Sl_SAS{ee}{ff}(relative_peak_pos+1:end) = [];
                tSl_SAS{ee}{ff}(relative_peak_pos+1:end) = [];
            end
        else
            Sl_SAS{ee}{ff} = [];
            tSl_SAS{ee}{ff} = [];
        end
    end
end


% 4.2. winnow superfluous empty cells

for ee = 1:length(Sl_SAS)
    for ff = 1:length(Sl_SAS{ee})
        Sl_SAS{ee}(cellfun('isempty',Sl_SAS{ee})) = [];
        tSl_SAS{ee}(cellfun('isempty',tSl_SAS{ee})) = [];
    end
end


for ee = 1:length(Sl_SAS)
    Sl_SAS(cellfun('isempty',Sl_SAS)) = [];
    tSl_SAS(cellfun('isempty',tSl_SAS)) = [];
end

% 4.3. calculate slope m

for ee = 1:length(Sl_SAS)
    for ff = 1:length(Sl_SAS{ee})
        for gg = 1:length(Sl_SAS{ee}{ff}) -1; % loop through individual pulses
            x{ee}{ff}(gg) = ((tSl_SAS{ee}{ff}(gg+1) - tSl_SAS{ee}{ff}(gg))/Fs)*1000; %time in ms
            y{ee}{ff}(gg) = Sl_SAS{ee}{ff}(gg+1) - Sl_SAS{ee}{ff}(gg);
            m_SAS{ee}{ff}(gg) = y{ee}{ff}(gg)/x{ee}{ff}(gg);
        end
    end
end

    
% 4.4. calculate average slope M
if isempty(Sl_SAS) == 1
    m_SAS = [];
end

if isempty(m_SAS) == 1 
    concat_M_SAS_h1{ee} = [];
    concat_M_SAS = [];
    concat_x = [];
    concat_y = [];
else
    for ee = 1:length(m_SAS)
        if isempty( m_SAS{ee}) == 1
            concat_M_SAS_h1{ee} = horzcat(m_SAS{ee});
            concat_x_h1{ee} = horzcat(x{ee});
            concat_y_h1{ee} = horzcat(y{ee});
        else
            concat_M_SAS_h1{ee} = horzcat(m_SAS{ee}{:});
            concat_x_h1{ee} = horzcat(x{ee}{:});
            concat_y_h1{ee} = horzcat(y{ee}{:});
        end
    end
    concat_M_SAS = horzcat(concat_M_SAS_h1{:});
    concat_x = horzcat(concat_x_h1{:});
    concat_y = horzcat(concat_y_h1{:});
end

% concat_M_SAS(find(concat_D_SAS_h_outlier > max_duration_SAS)) = [];
concat_M_SAS(find(isnan(concat_M_SAS) == 1)) = [];
if max(concat_M_SAS) >1
    concat_M_SAS(find(concat_M_SAS > 1)) = 1;
end

M_SAS = sum(concat_M_SAS)/length(concat_M_SAS);
M_SAS_std = std(concat_M_SAS);
M_SAS_se = M_SAS_std/sqrt(length(concat_M_SAS));
    
end